module COREFLAGGEN_TIP (
                        input           gsr_b,  // async active low reset
                        input           sclk0,  // [0] for RX, divclk from lanectrl
                        input           stream_start,  // stream start from SGMII IP
                        input           clr_flags_n,  // active high reset from CDR control
                        input [9:0]     rx_p,  // RX data from IOGP
                        input [9:0]     rx_n,  // RX data from IOGN
                        output reg      load_n,  // load flash bits to INDLY
                        output reg      move_n,  // move INDLY up/down on rising edge
                        output reg      direction_n,  // 1=up, 0=down
                        output reg      flag_ready,  // initial flag ready to CDR control
                        output reg      early_n,  // active high early flag to CDR control
                        output reg      late_n);  // active high late flag to CDR control

parameter       ERR_NUM = 1;  // accumulated number of errors
parameter       OFFSET = 4'b0111;  // early or late window size

reg [1:0]       rst_b;  // sync-off internal reset on sclk0 domain
reg [1:0]       state;  // state machine
reg [4:0]       cnt;  // move/wait counter
reg [2:0]       err_cnt;  // mismatch error counter
reg             clr_flags_n_sync;

wire reset_b = rst_b[0];
wire mismatch = (rx_p != rx_n);  // RX data does not match with early or late data
wire clr_flags = clr_flags_n & !clr_flags_n_sync;  // one cycle pulse reset
wire err_flag = mismatch && (err_cnt == ERR_NUM);  // assert flags

always @(posedge sclk0 or negedge gsr_b) begin  // RX or non-stop clock
   if (!gsr_b) rst_b <= 0;  // async-on
   else if (stream_start) rst_b <= {1'b1, rst_b[1]};  // sync-off after stream starts
end

always @(posedge sclk0 or negedge reset_b)
   if (!reset_b) clr_flags_n_sync <= 1'b0;
   else clr_flags_n_sync <= clr_flags_n;

always @(posedge sclk0 or negedge reset_b) begin  // RX clock
   if (!reset_b) begin
      flag_ready <= 1'b0;
      early_n <= 1'b0;
      late_n <= 1'b0;
      load_n <= 1'b0;
      move_n <= 1'b0;
      direction_n <= 1'b0;  // begin with early flag side
      cnt <= 0;
      state <= 0;
      err_cnt <= 0;
   end
   else if (clr_flags) begin  // at each nxtclk update, reset
      early_n <= 1'b0;
      late_n <= 1'b0;
      load_n <= 1'b0;
      move_n <= 1'b0;
      cnt <= 0;
      state <= 0;
      err_cnt <= 0;
      if (flag_ready) direction_n <= ~direction_n;  // keep checking the same side after nxtclk update
   end
   else case (state)
      2'b00 : begin  // load state
                 if (&cnt[2:0]) begin  // wait eight cycles in case of flag raised at the end of last check
                    state <= 2'b01;  // to move state 
                    cnt <= 0;
                    err_cnt <= 0;
                    load_n <= 1'b1;  // load IOGN flash bits, which has to be the same as IOGP 
                    direction_n <= ~direction_n;  // checking the opposite side
                 end
                 else cnt <= cnt + 1;
              end
      2'b01 : begin  // move state
                 load_n <= 1'b0;  // one cycle assertion
                 if (cnt[0]) begin
                    if (cnt[4:1] == OFFSET) begin // after moved to the boundary
                       state <= 2'b10;  // to wait state
                       cnt <= 0;
                    end 
                    else begin
                       cnt <= cnt + 1;
                       move_n <= 1'b1;  // load DLL code to RXDLY
                    end 
                 end
                 else begin
                    cnt <= cnt + 1;
                    move_n <= 1'b0;  // one cycle assertion
                 end
              end
      2'b10 : begin  // wait state
                 if (&cnt[1:0]) begin  // wait four cycles
                    state <= 2'b11;  // after rx_n flush clean, to check state
                    cnt <= 0;
                 end
                 else cnt <= cnt + 1;
              end
      2'b11 : begin  // check state
                 if (&cnt) state <= 2'b00;  // after counter max, to load state
                 cnt <= cnt + 1;
                 if (mismatch) err_cnt <= err_cnt + 1;  // mismatch errors

                 if (direction_n && (err_flag || (&cnt))) flag_ready <= 1'b1;  // sticky flag ready after both flags generated
                 if (!direction_n & err_flag) early_n <= 1'b1;  // sticky early flag
                 if (direction_n & err_flag) late_n <= 1'b1;  // sticky late flag
              end
   endcase
end

endmodule